package app;

import gui.CharacterFrame;
import gui.RegisterFrame;
import gui.LoginFrame;
import gui.WelcomeFrame;
import jpa.CharactersEntity;
import jpa.ServersEntity;
import jpa.StoresEntity;
import jpa.UsersEntity;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.hibernate.exception.ConstraintViolationException;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;

import javax.swing.*;
import java.sql.Timestamp;
import java.util.Collection;
import java.util.Date;

/**
 * Created by Wouter on 10/5/2015.
 */
public class Main {
    private static final SessionFactory ourSessionFactory;
    private static final ServiceRegistry serviceRegistry;
    private JFrame currentFrame;
    public static final int LOGINFRAME = 0;
    public static final int REGISTERFRAME = 1;
    public static final int CHARACTERFRAME = 2;
    public static final int WELCOMEFRAME = 3;
    private UsersEntity userLoggedIn;

    private String[] classes = new String[]{
            "HADOOPMAGE",
            "NOSOLDIERQL",
            "SWEQMASTER",
            "HOGESCHOOLLERAAR",
            "ROOSTEMAKER",
            "KOFFIEPROEVER",
            "DUBBELFRIS",
            "BAKKIEPLEUR",
            "RACEAUTO",
            "DIKKEBMW",
            "DERPSLET",
            "TEQPLAYA"

    };
    private String[] races = new String[]{
            "MENS",
            "NOTZI",
            "MOK",
            "BRANDWEERAUTO",
            "FRUITVLIEG"
    };
    static {
        try {
            Configuration configuration = new Configuration();
            configuration.configure();

            serviceRegistry = new ServiceRegistryBuilder().applySettings(configuration.getProperties()).buildServiceRegistry();
            ourSessionFactory = configuration.buildSessionFactory(serviceRegistry);
        } catch (Throwable ex) {
            throw new ExceptionInInitializerError(ex);
        }
    }

    public static Session getSession() throws HibernateException {
        return ourSessionFactory.openSession();
    }

    public static void main(final String[] args) throws Exception {
        new Main();
    }
    public Main(){
        currentFrame = new LoginFrame(this);
        currentFrame.setVisible(true);
//        final Session session = getSession();
//        try {
//            Transaction tx = session.beginTransaction();
//            UsersEntity user = UsersEntity.getUser(session,"wouter");
//            Random rand = new Random();
//            int balance = rand.nextInt(100);
//            System.out.println(user.getUserName());
//            System.out.println(user.getBalance());
//            System.out.println(balance);
//            user.setBalance(balance);
//            UsersEntity user2 = new UsersEntity("sjors","test");
//            session.save(user2);
//            session.update(user);
//            tx.commit();
//        } finally {
//            session.close();
//        }
    }
    public void switchFrame(int frame){
        currentFrame.dispose();
        if(frame == LOGINFRAME){
            currentFrame = new LoginFrame(this);
        }else if(frame == REGISTERFRAME){
            currentFrame = new RegisterFrame(this);
        }
        else if(frame == CHARACTERFRAME){
            currentFrame = new CharacterFrame(this);
        }
        else if(frame == WELCOMEFRAME){
            currentFrame = new WelcomeFrame(this);
        }
        else{
            currentFrame = new LoginFrame(this);
        }
        currentFrame.setVisible(true);
    }
    public void connectToServer(CharactersEntity charactersEntity){
        final Session session = getSession();
        boolean loggedIn = false;
        try{
            Transaction tx = session.beginTransaction();

            ServersEntity server;
            String address = charactersEntity.getServer();
            final org.hibernate.Query query = session.createQuery("from jpa.ServersEntity where address = :address");
            query.setString("address",address);
            if(query.list().size() > 0){
                server = (ServersEntity)query.list().get(0);
                if(server.getConnectedUsers() < server.getMaxUsers()){
                    StoresEntity store = new StoresEntity();
                    store.setAddress(address);
                    store.setCharacterName(charactersEntity.getName());
                    server.setConnectedUsers(server.getConnectedUsers() + 1);
                    session.save(store);
                    loggedIn = true;
                    JOptionPane.showMessageDialog(null, "connection to server was successful!", "Succes", JOptionPane.INFORMATION_MESSAGE);
                }else{
                    JOptionPane.showMessageDialog(null, "server is full", "Error", JOptionPane.INFORMATION_MESSAGE);
                    throw  new Exception("server is full");
                }
            }else{
                JOptionPane.showMessageDialog(null, "server not found", "Error", JOptionPane.INFORMATION_MESSAGE);
                throw new Exception("server not found");
            }

            tx.commit();
        }
        catch (ConstraintViolationException constraintViolationException){
            JOptionPane.showMessageDialog(null, "Character already logged in", "Error", JOptionPane.INFORMATION_MESSAGE);
        }
        catch (Exception e){
        }
        finally {
            session.close();
        }
        if(loggedIn){
            switchFrame(WELCOMEFRAME);
        }
    }
    public UsersEntity getUser(Session session,String user_name){
        final Query query = session.createQuery("from jpa.UsersEntity where userName = :userName");
        query.setString("userName",user_name);
        if(query.list().size() > 0)
            return (UsersEntity)query.list().get(0);
        return null;
    }
    public CharactersEntity getLoggedinCharacterByUser() {
        final Session session = getSession();
        CharactersEntity loggedInCharacter = null;
        try {
            Collection<CharactersEntity> characters = getUserLoggedIn().getCharactersesByUserName();
            for (CharactersEntity character : characters) {
                final org.hibernate.Query query = session.createQuery("from jpa.StoresEntity where characterName = :characterName");
                query.setString("characterName", character.getName());
                if (query.list().size() > 0) {
                    loggedInCharacter = character;
                }
            }
        }
        finally {
            session.close();
        }
        return loggedInCharacter;
    }
    public void logoutCharacter(CharactersEntity charactersEntity) {
        final Session session = getSession();
        try{
            Transaction tx = session.beginTransaction();
            ServersEntity server;
            String address = charactersEntity.getServer();
            final org.hibernate.Query query = session.createQuery("delete from jpa.StoresEntity where characterName = :characterName");
            query.setString("characterName",charactersEntity.getName());
            query.executeUpdate();
            final org.hibernate.Query serverquery = session.createQuery("from jpa.ServersEntity where address = :address");
            serverquery.setString("address",address);
            if(serverquery.list().size() > 0){
                server = (ServersEntity)serverquery.list().get(0);
                server.setConnectedUsers(server.getConnectedUsers()-1);
            }
            tx.commit();
            }
        finally {
            session.close();
        }
    }
    public void addMoney(CharacterFrame frame){
        final Session session = getSession();
        try {
            Transaction tx = session.beginTransaction();
            //get user balance
            UsersEntity currentUser = getUserLoggedIn();
            int currentBalance = 0;
            currentBalance = currentUser.getBalance();
            int spinnerValue = (Integer)frame.add_money_spinner.getValue();
            if(spinnerValue>0) {
                currentBalance += spinnerValue;
            }

            //set new balance
            currentUser.setBalance(currentBalance);

            //show new balance
            currentBalance = currentUser.getBalance();
            frame.balance.setText(currentBalance + "$");

            session.update(currentUser);
            tx.commit();
        }catch(ConstraintViolationException constrainViolation) {
        }catch (Exception normalException){

        }
        finally {
            session.close();
        }
    }
    public void renovating_subscription(CharacterFrame frame){
        final Session session = getSession();
        try {
            Transaction tx = session.beginTransaction();

            int price1Month = 5;
            int price2Months = 9;
            int price3Months = 13;
            int price1Year = 25;

            //get user balance
            UsersEntity currentUser = getUserLoggedIn();
            int currentBalance = currentUser.getBalance();
            int currentMonthsPayed = currentUser.getMonthsPayed();


            String subscriptionComboboxValue = (String)frame.renovating_subscription_combobox_month_selection.getSelectedItem();
            if(subscriptionComboboxValue.equals("1 month")) {
                if(currentBalance>=price1Month) {
                    currentUser.setBalance(currentBalance-price1Month);
                    currentUser.setMonthsPayed(currentMonthsPayed + 1);

                    //get current timestamp
                    Date dateTime = new Date();
                    Timestamp currentTimeStamp = new Timestamp((dateTime).getTime());
                    currentUser.setLastPayment(currentTimeStamp);
                    //show current months
                    frame.error_box.setText("Months payed: " + currentUser.getMonthsPayed());
                }
                else {
                    frame.error_box.setText("Please load more monies into your account plsthnx");
                }
            }
            else if(subscriptionComboboxValue.equals("2 months")) {
                if(currentBalance>=price2Months) {
                    currentUser.setBalance(currentBalance-price2Months);
                    currentUser.setMonthsPayed(currentMonthsPayed + 2);

                    //get current timestamp
                    Date dateTime = new Date();
                    Timestamp currentTimeStamp = new Timestamp((dateTime).getTime());
                    currentUser.setLastPayment(currentTimeStamp);
                    //show current months
                    frame.error_box.setText("Months payed: " + currentUser.getMonthsPayed());
                }
                else {
                    frame.error_box.setText("Please load more monies into your account plsthnx");
                }
            }
            else if(subscriptionComboboxValue.equals("3 months")) {
                if(currentBalance>=price3Months) {
                    currentUser.setBalance(currentBalance-price3Months);
                    currentUser.setMonthsPayed(currentMonthsPayed + 3);

                    //get current timestamp
                    Date dateTime = new Date();
                    Timestamp currentTimeStamp = new Timestamp((dateTime).getTime());
                    currentUser.setLastPayment(currentTimeStamp);
                    //show current months
                    frame.error_box.setText("Months payed: " + currentUser.getMonthsPayed());
                }
                else {
                    frame.error_box.setText("Please load more monies into your account plsthnx");
                }
            }
            else if(subscriptionComboboxValue.equals("1 year")) {
                if(currentBalance>=price1Year) {
                    currentUser.setBalance(currentBalance-price1Year);
                    currentUser.setMonthsPayed(currentMonthsPayed + 12);

                    //get current timestamp
                    Date dateTime = new Date();
                    Timestamp currentTimeStamp = new Timestamp((dateTime).getTime());
                    currentUser.setLastPayment(currentTimeStamp);
                    //show current months
                    frame.error_box.setText("Months payed: " + currentUser.getMonthsPayed());
                }
                else {
                    frame.error_box.setText("Please load more monies into your account plsthnx");
                }
            }

            //show new balance
            currentBalance = currentUser.getBalance();
            frame.balance.setText(currentBalance + "$");

            session.update(currentUser);
            tx.commit();
        }catch(ConstraintViolationException constrainViolation) {
        }catch (Exception normalException){

        }
        finally {
            session.close();
        }
    }
    public void buyCharSlots(CharacterFrame frame){
        final Session session = getSession();
        try {
            Transaction tx = session.beginTransaction();
            //get user balance
            UsersEntity currentUser = getUserLoggedIn();
            int currentBalance = currentUser.getBalance();
            int currentCharacterSlots = currentUser.getCharacterSlots();
            int spinnerValue = (Integer)frame.character_slots_spinner.getValue();
            int slotPrice = 5;
            if(spinnerValue>0) {
                if(currentBalance >= spinnerValue*slotPrice){
                    currentBalance -= (spinnerValue*slotPrice);
                    //set new character slots
                    currentUser.setCharacterSlots(currentCharacterSlots+spinnerValue);
                    //update character slots
                    currentCharacterSlots = currentUser.getCharacterSlots();
                    frame.error_box.setText("current slots:" + currentCharacterSlots);
                }else{
                    frame.error_box.setText("Plz need more munnie you hav nut enuf, por peple r nut alowed to ple tooronli");
                }
            }

            //set new balance
            currentUser.setBalance(currentBalance);

            //show new balance
            currentBalance = currentUser.getBalance();
            frame.balance.setText(currentBalance + "$");

            session.update(currentUser);
            tx.commit();
        }catch(ConstraintViolationException constrainViolation) {
        }catch (Exception normalException){

        }finally {
            session.close();
        }
    }
    public ServersEntity[] getServers(){
        final Session session = getSession();
        ServersEntity[] servers;
        try{
            Transaction tx = session.beginTransaction();
            final org.hibernate.Query serverquery = session.createQuery("from jpa.ServersEntity");
            servers = new ServersEntity[serverquery.list().size()];
            for (int serverIndex = 0; serverIndex < serverquery.list().size(); serverIndex++) {
                servers[serverIndex] = (ServersEntity)serverquery.list().get(serverIndex);
            }
            tx.commit();
        }
        finally {
            session.close();
        }
        return servers;
    }
    public UsersEntity getUserLoggedIn(){
        return userLoggedIn;
    }
    public void setUserLoggedIn(UsersEntity user){
        userLoggedIn = user;
    }

    public String[] getClasses() {
        return classes;
    }

    public void setClasses(String[] classes) {
        this.classes = classes;
    }

    public String[] getRaces() {
        return races;
    }

    public void setRaces(String[] races) {
        this.races = races;
    }
}
